import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from werkzeug.exceptions import InternalServerError

from flaschengeist import logger
from flaschengeist.models import User
from flaschengeist.plugins import Plugin
from flaschengeist.utils.hook import HookAfter
from flaschengeist.controller import userController
from flaschengeist.controller.messageController import Message
from flaschengeist.config import config


class MailMessagePlugin(Plugin):
    def load(self):
        self.config = config.get("mail", None)
        if self.config is None:
            logger.error("mail was not configured in flaschengeist.toml")
            raise InternalServerError
        self.server = self.config["SERVER"]
        self.port = self.config["PORT"]
        self.user = self.config["USER"]
        self.password = self.config["PASSWORD"]
        self.crypt = self.config["CRYPT"]
        self.mail = self.config["MAIL"]

        @HookAfter("send_message")
        def dummy_send(msg, *args, **kwargs):
            logger.info(f"(dummy_send) Sending message to {msg.receiver}")
            self.send_mail(msg)

    def send_mail(self, msg: Message):
        logger.debug(f"Sending mail to {msg.receiver} with subject {msg.subject}")
        if isinstance(msg.receiver, User):
            if not msg.receiver.mail:
                logger.warning("Could not send Mail, mail missing: {}".format(msg.receiver))
                return
            recipients = [msg.receiver.mail]
        else:
            recipients = userController.get_user_by_role(msg.receiver)

        mail = MIMEMultipart()
        mail["From"] = self.mail
        mail["To"] = ", ".join(recipients)
        mail["Subject"] = msg.subject
        mail.attach(MIMEText(msg.message))
        with self.__connect() as smtp:
            smtp.sendmail(self.mail, recipients, mail.as_string())

    def __connect(self):
        if self.crypt == "SSL":
            self.smtp = smtplib.SMTP_SSL(self.server, self.port)
        elif self.crypt == "STARTTLS":
            self.smtp = smtplib.SMTP(self.server, self.port)
            self.smtp.starttls()
        else:
            raise ValueError("Invalid CRYPT given")
        self.smtp.login(self.user, self.password)
        return self.smtp
